@@ -106,8 +106,6 @@ GArray *_atspi_dbus_return_attribute_array_from_message (DBusMessage *message);
 
 GArray *_atspi_dbus_attribute_array_from_iter (DBusMessageIter *iter);
 
-gboolean _atspi_process_deferred_messages (gpointer data);
-
 void _atspi_dbus_set_interfaces (AtspiAccessible *accessible, DBusMessageIter *iter);
 
 void _atspi_dbus_set_state (AtspiAccessible *accessible, DBusMessageIter *iter);
@@ -149,7 +147,8 @@ GQuark _atspi_error_quark (void);
 typedef enum
 {
   ATSPI_ERROR_APPLICATION_GONE,
-  ATSPI_ERROR_IPC
+  ATSPI_ERROR_IPC,
+  ATSPI_ERROR_SYNC_NOT_ALLOWED,
 } AtspiError;
 
 extern GMainLoop *atspi_main_loop;
@@ -158,6 +157,14 @@ extern gboolean atspi_no_cache;
 GHashTable *_atspi_get_live_refs ();
 
 gchar *_atspi_name_compat (gchar *in);
+
+void _atspi_dbus_update_cache_from_dict (AtspiAccessible *accessible, DBusMessageIter *iter);
+
+gboolean _atspi_get_allow_sync ();
+
+gboolean _atspi_set_allow_sync (gboolean val);
+
+void _atspi_set_error_no_sync (GError **error);
 G_END_DECLS
 
 #endif	/* _ATSPI_MISC_PRIVATE_H_ */
